var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { useMutation } from '@tanstack/react-query';
import { BigNumber as BigNumberEth } from 'ethers';
import { errorMap, ErrorFrom } from '../utils/errorMap';
import { useWeb3LibraryContext } from '../wagmiUtil';
import { useActiveWeb3React } from './useActiveWeb3React';
import { toast } from 'react-toastify';
import { TransactionStatus, useUpsertTransaction } from '../providers/TransactionProvider';
import { Connection, VersionedTransaction } from '@solana/web3.js';
import { useConnection } from '@solana/wallet-adapter-react';
import { isSolChain } from '../utils/chainIdUtil';
import { postTransaction } from '../utils/postTransaction';
import BigNumber from 'bignumber.js';
import { useConfig } from '../providers/ConfigProviders';
import { solConnectionPools } from '../providers/SolanaProvider';
import { getSolSimulationError } from '../components/SolanaConnector/SolanaConstant';
import { useWETH } from './useContract';
import { ChainId } from '../utils/constants';
/**
 * Get swap handler functions and swap transaction state
 */
export const useSwapCallback = () => {
    var _a, _b;
    const { account, solanaWallet, chainId } = useActiveWeb3React();
    const { connection: solanaConnection } = useConnection();
    const library = useWeb3LibraryContext();
    const wethContract = useWETH(chainId, true);
    const [config] = useConfig();
    const affiliateData = {
        marketer: config.marketer,
        affiliateFee: Number(config.affiliateFee),
        affiliateFeeRecipient: (_b = (_a = config.affiliateFeeRecipientByChains) === null || _a === void 0 ? void 0 : _a[chainId]) !== null && _b !== void 0 ? _b : config.affiliateFeeRecipient,
    };
    const upsertTransaction = useUpsertTransaction();
    const swapMutation = useMutation((_c) => __awaiter(void 0, [_c], void 0, function* ({ quote, inputToken, outputToken, formattedInputAmount, formattedOutputAmount, inputTokenUSD, }) {
        const payload = {
            quote,
            inputToken,
            outputToken,
            formattedInputAmount,
            formattedOutputAmount,
            inputTokenUSD,
            account: account,
            affiliateData,
            upsertTransaction,
            library: library,
            solanaWallet,
            solanaConnection,
            wethContract,
        };
        if (isSolChain(quote.chainId)) {
            return yield handleSolSwap(payload);
        }
        if (quote.wrapUnwrap) {
            return yield handleEvmWrapUnwrap(payload);
        }
        return yield handleEvmSwap(payload);
    }), {
        onError: (error, vars) => {
            let msg = '';
            // if the user rejected the tx, pass this along
            if ((error === null || error === void 0 ? void 0 : error.code) === 4001) {
                msg = 'Transaction rejected.';
            }
            else {
                // otherwise, the error was unexpected and we need to convey that
                console.error(`Swap failed`, error, vars.quote);
                msg = `Swap failed: ${errorMap(error.message, ErrorFrom.SWAP)}`;
            }
            toast.error(msg);
        },
        mutationKey: ['swap'],
    });
    return swapMutation;
};
const handleSolSwap = (_a) => __awaiter(void 0, [_a], void 0, function* ({ inputToken, outputToken, quote, account, formattedInputAmount, formattedOutputAmount, inputTokenUSD, affiliateData, solanaConnection, solanaWallet, upsertTransaction, }) {
    var _b, _c;
    const inputSymbol = inputToken.symbol;
    const outputSymbol = outputToken.symbol;
    const swapTransactionBuf = Buffer.from(quote.data, 'base64');
    const solTransaction = VersionedTransaction.deserialize(swapTransactionBuf);
    const sim = yield solanaConnection.simulateTransaction(solTransaction);
    const simError = getSolSimulationError(sim);
    if (simError) {
        // show a better error message?
        throw new Error(`Your transaction cannot be completed: ${simError}`);
    }
    const sendTransactionOptions = {
        skipPreflight: true,
        maxRetries: 2,
        preflightCommitment: 'confirmed',
    };
    let txid = '';
    if (((_c = (_b = solanaWallet.wallet) === null || _b === void 0 ? void 0 : _b.adapter) === null || _c === void 0 ? void 0 : _c.name) === 'Phantom') {
        // sign and send transaction in one step for Phantom wallet
        txid = yield solanaWallet.sendTransaction(solTransaction, solanaConnection, sendTransactionOptions);
    }
    else {
        const signedTransaction = yield solanaWallet.signTransaction(solTransaction);
        // Send the signed transaction using multiple rpc endpoints to maximize the chance of success
        txid = yield Promise.any(solConnectionPools.map((ep) => new Connection(ep).sendTransaction(signedTransaction, sendTransactionOptions)));
    }
    const transaction = {
        status: TransactionStatus.Pending,
        txn: txid,
        message: `Swap ${formattedInputAmount} ${inputSymbol} for ${formattedOutputAmount} ${outputSymbol}`,
        inputToken,
        outputToken,
        quote,
        chainId: quote.chainId,
        account: account,
        createdAt: new Date(),
    };
    const dbtransaction = {
        txn: txid,
        maker: account,
        sellToken: inputToken.address,
        buyToken: outputToken.address,
        dollarValue: inputTokenUSD === null || inputTokenUSD === void 0 ? void 0 : inputTokenUSD.decimalPlaces(3, BigNumber.ROUND_CEIL).toNumber(),
        chainId: quote === null || quote === void 0 ? void 0 : quote.chainId,
        createdAt: new Date(),
        sellTokenValue: formattedInputAmount,
        buyTokenValue: formattedOutputAmount,
        marketer: affiliateData.marketer,
        affiliateFee: affiliateData.affiliateFee,
        affiliateFeeRecipient: affiliateData.affiliateFeeRecipient,
    };
    postTransaction(dbtransaction);
    upsertTransaction(transaction);
    return transaction;
});
const handleEvmSwap = (_d) => __awaiter(void 0, [_d], void 0, function* ({ inputToken, outputToken, quote, account, formattedInputAmount, formattedOutputAmount, inputTokenUSD, affiliateData, library, upsertTransaction, }) {
    const inputSymbol = inputToken.symbol;
    const outputSymbol = outputToken.symbol;
    let gas = undefined;
    if (quote.gasPrice) {
        if (quote.chainId === ChainId.ETH) {
            // ignore gas padding for ETH
            gas = BigNumberEth.from(quote.gas).toHexString();
        }
        else {
            // add ~20% gas just to be safe
            gas = BigNumberEth.from(quote.gas).mul(12).div(10).toHexString();
        }
    }
    const transactionHash = yield library.send('eth_sendTransaction', [
        {
            from: account,
            to: quote.to,
            gas,
            gasPrice: BigNumberEth.from(quote.gasPrice).toHexString(),
            value: BigNumberEth.from(quote.value).toHexString(),
            data: quote.data,
        },
    ]);
    if (!transactionHash) {
        throw Error('No transaction hash found.');
    }
    const transaction = {
        status: TransactionStatus.Pending,
        txn: transactionHash,
        message: `Swap ${formattedInputAmount} ${inputSymbol} for ${formattedOutputAmount} ${outputSymbol}`,
        inputToken,
        outputToken,
        quote,
        chainId: quote.chainId,
        account: account,
        createdAt: new Date(),
    };
    const dbtransaction = {
        txn: transactionHash,
        maker: account,
        sellToken: inputToken.address,
        buyToken: outputToken.address,
        dollarValue: inputTokenUSD === null || inputTokenUSD === void 0 ? void 0 : inputTokenUSD.decimalPlaces(3, BigNumber.ROUND_CEIL).toNumber(),
        chainId: quote === null || quote === void 0 ? void 0 : quote.chainId,
        createdAt: new Date(),
        sellTokenValue: formattedInputAmount,
        buyTokenValue: formattedOutputAmount,
        marketer: affiliateData.marketer,
        affiliateFee: affiliateData.affiliateFee,
        affiliateFeeRecipient: affiliateData.affiliateFeeRecipient,
    };
    postTransaction(dbtransaction);
    upsertTransaction(transaction);
    return transaction;
});
const handleEvmWrapUnwrap = (_e) => __awaiter(void 0, [_e], void 0, function* ({ inputToken, outputToken, quote, account, formattedInputAmount, formattedOutputAmount, wethContract, upsertTransaction, }) {
    const inputSymbol = inputToken.symbol;
    const outputSymbol = outputToken.symbol;
    const transactionOpts = {
        // add ~20% gas just to be safe
        gasPrice: BigNumberEth.from(quote.gasPrice).toHexString(),
        value: BigNumberEth.from(quote.value).toHexString(),
    };
    const ethTransaction = yield (quote.wrapUnwrap === 'Wrap'
        ? wethContract.deposit(transactionOpts)
        : wethContract.withdraw(transactionOpts.value, {
            gasPrice: transactionOpts.gasPrice,
        }));
    const transactionHash = ethTransaction.hash;
    if (!transactionHash) {
        throw Error('No transaction hash found.');
    }
    const transaction = {
        status: TransactionStatus.Pending,
        txn: transactionHash,
        message: `Swap ${formattedInputAmount} ${inputSymbol} for ${formattedOutputAmount} ${outputSymbol}`,
        inputToken,
        outputToken,
        quote,
        chainId: quote.chainId,
        account: account,
        createdAt: new Date(),
    };
    upsertTransaction(transaction);
    return transaction;
});
//# sourceMappingURL=useSwapCallback.js.map