import BigNumber from 'bignumber.js';
import { wrappedToken } from '../utils';
import { STABLE_COINS } from '../utils/multichain';
import { useQuery } from '@tanstack/react-query';
import { useActiveWeb3React } from './useActiveWeb3React';
import { useOffChainOracleContract } from './useContract';
import { useToken } from './useToken';
import { BIG_18D } from '../utils/constants';
import React, { useMemo } from 'react';
import { useJupiterPrice } from './useJupiterPrice';
import { isCustomChain, isSolChain } from '../utils/chainIdUtil';
export const useEthUsdRate = () => {
    var _a;
    const { chainId } = useActiveWeb3React();
    const contract = useOffChainOracleContract();
    const stable = (_a = STABLE_COINS[chainId]) === null || _a === void 0 ? void 0 : _a[0];
    const enabled = Boolean(chainId && !isSolChain(chainId) && stable);
    const { data } = useQuery(['ethUsdPrice', stable === null || stable === void 0 ? void 0 : stable.address], () => contract.getRateToEth(stable === null || stable === void 0 ? void 0 : stable.address, true).then((ethUsdRate) => {
        const numerator = new BigNumber(10).pow(stable === null || stable === void 0 ? void 0 : stable.decimals);
        const priceUsd = new BigNumber(ethUsdRate._hex).times(numerator).div(BIG_18D);
        return priceUsd;
    }), {
        enabled,
        refetchOnMount: false,
        refetchOnWindowFocus: true,
        refetchInterval: 30 * 1000, // every 30s
    });
    return enabled ? data : null;
};
/**
 * Get the USD price of a token
 * @param token The token to get the price of
 * @param stableToken The optional stable token to compare the price in
 * @returns The price of the token in BigNumber (decimal parsed). Returns null if empty or loading
 */
export const useUSDPrice = (token) => {
    const { chainId } = useActiveWeb3React();
    const contract = useOffChainOracleContract();
    const ethUsd = useEthUsdRate();
    const isSol = isSolChain(chainId);
    const jupiterPrice = useJupiterPrice(token === null || token === void 0 ? void 0 : token.address, chainId);
    const enabled = Boolean(token && chainId && !isSol && ethUsd);
    const wrapped = wrappedToken(token);
    const { data } = useQuery(['usdPrice', token, ethUsd], () => contract.getRateToEth(wrapped === null || wrapped === void 0 ? void 0 : wrapped.address, true).then((tokenEthRate) => {
        const numerator = new BigNumber(10).pow(token === null || token === void 0 ? void 0 : token.decimals).div(ethUsd);
        const tokenUsd = new BigNumber(tokenEthRate._hex).times(numerator).div(BIG_18D);
        return tokenUsd;
    }), {
        enabled,
        refetchOnMount: false,
        refetchOnWindowFocus: false,
    });
    return isSol ? jupiterPrice : enabled ? data : null;
};
/**
 * Get the USD price of a token by address
 * @param address The address of the token to get the price of
 * @returns The price of the token in BigNumber (decimal parsed). Returns null if empty or loading
 */
export const useUSDPriceByAddress = (address) => {
    const token = useToken(address);
    return useUSDPrice(token);
};
export const useSwapDerivedUSDPrice = (tokenToEthRate) => {
    const { chainId } = useActiveWeb3React();
    const ethUsd = useEthUsdRate();
    return React.useMemo(() => {
        if (tokenToEthRate && isCustomChain(chainId) && ethUsd) {
            const rate = new BigNumber(tokenToEthRate);
            const ethRate = new BigNumber(10).pow(18).div(ethUsd);
            return ethRate.div(rate);
        }
        return null;
    }, [tokenToEthRate, chainId, ethUsd]);
};
export const useInputOutputUsdPrice = (priceQuote, formattedInputAmount, formattedOutputAmount) => {
    const inputTokenDerivedUSDPrice = useSwapDerivedUSDPrice(priceQuote === null || priceQuote === void 0 ? void 0 : priceQuote.sellTokenToEthRate);
    const outputTokenDerivedUSDPrice = useSwapDerivedUSDPrice(priceQuote === null || priceQuote === void 0 ? void 0 : priceQuote.buyTokenToEthRate);
    const inputTokenDerivedUSDPriceSol = useJupiterPrice(priceQuote === null || priceQuote === void 0 ? void 0 : priceQuote.sellTokenAddress, priceQuote === null || priceQuote === void 0 ? void 0 : priceQuote.chainId);
    const outputTokenDerivedUSDPriceSol = useJupiterPrice(priceQuote === null || priceQuote === void 0 ? void 0 : priceQuote.buyTokenAddress, priceQuote === null || priceQuote === void 0 ? void 0 : priceQuote.chainId);
    return useMemo(() => [
        // input
        inputTokenDerivedUSDPriceSol
            ? inputTokenDerivedUSDPriceSol.times(formattedInputAmount)
            : outputTokenDerivedUSDPriceSol
                ? outputTokenDerivedUSDPriceSol
                    .times(formattedOutputAmount)
                    .div((100 - parseFloat(priceQuote === null || priceQuote === void 0 ? void 0 : priceQuote.estimatedPriceImpact)) / 100 || 1)
                : (priceQuote === null || priceQuote === void 0 ? void 0 : priceQuote.buyTokenToUsdRate)
                    ? new BigNumber(priceQuote.buyTokenToUsdRate)
                    : inputTokenDerivedUSDPrice === null || inputTokenDerivedUSDPrice === void 0 ? void 0 : inputTokenDerivedUSDPrice.times(formattedInputAmount),
        // output
        outputTokenDerivedUSDPriceSol
            ? outputTokenDerivedUSDPriceSol.times(formattedOutputAmount)
            : inputTokenDerivedUSDPriceSol
                ? inputTokenDerivedUSDPriceSol
                    .times(formattedInputAmount)
                    .times((100 - parseFloat(priceQuote === null || priceQuote === void 0 ? void 0 : priceQuote.estimatedPriceImpact)) / 100 || 1)
                : (priceQuote === null || priceQuote === void 0 ? void 0 : priceQuote.buyTokenToUsdRate)
                    ? new BigNumber(priceQuote.buyTokenToUsdRate)
                    : outputTokenDerivedUSDPrice === null || outputTokenDerivedUSDPrice === void 0 ? void 0 : outputTokenDerivedUSDPrice.times(formattedOutputAmount),
    ], [
        formattedInputAmount,
        formattedOutputAmount,
        inputTokenDerivedUSDPrice,
        inputTokenDerivedUSDPriceSol,
        outputTokenDerivedUSDPrice,
        outputTokenDerivedUSDPriceSol,
        priceQuote === null || priceQuote === void 0 ? void 0 : priceQuote.buyTokenToUsdRate,
        priceQuote === null || priceQuote === void 0 ? void 0 : priceQuote.estimatedPriceImpact,
    ]);
};
//# sourceMappingURL=useUSDPrice.js.map