import BigNumber from 'bignumber.js';
import { useQuery } from '@tanstack/react-query';
import { useActiveWeb3React } from './useActiveWeb3React';
import { useERC20 } from './useContract';
import { getBalanceAmount, isEth } from '../utils';
import { ETH, SOL_ADDR } from '../utils/multichain';
import { useToken } from './useToken';
import { useConnection } from '@solana/wallet-adapter-react';
import { BIG_ZERO } from '../utils/constants';
import { LAMPORTS_PER_SOL } from '@solana/web3.js';
import { SOLANA_TOKEN_PROGRAM } from '../components/SolanaConnector/SolanaConstant';
import { isSolChain } from '../utils/chainIdUtil';
/**
 * Get the wallet balance of a token
 * @param token The token to get the balance of
 * @returns The balance of the token in BigNumber (decimal parsed). Returns null if empty or loading
 */
export const useWalletBalance = (token) => {
    var _a;
    const { account, provider, chainId, solanaWallet } = useActiveWeb3React();
    const { connection } = useConnection();
    const ercContract = useERC20(token === null || token === void 0 ? void 0 : token.address, false);
    const solBalances = useAllSolTokenBalance();
    const isNotSolOrIsNativeSol = !isSolChain(chainId) || (token === null || token === void 0 ? void 0 : token.address) === SOL_ADDR;
    // disabled when it the chain is SOL unless the token is the native sol token
    // the rest sol token balances can be get from the result of useAllSolTokenBalance
    const enabled = Boolean(token && account && isNotSolOrIsNativeSol);
    const { data } = useQuery(['walletBalance', token === null || token === void 0 ? void 0 : token.chainId, token === null || token === void 0 ? void 0 : token.address], () => {
        if (isSolChain(token === null || token === void 0 ? void 0 : token.chainId)) {
            return connection.getBalance(solanaWallet.publicKey).then((response) => {
                return new BigNumber(response / LAMPORTS_PER_SOL);
            });
        }
        return (isEth(token.address, chainId) ? provider.getBalance(account) : ercContract.balanceOf(account)).then((bg) => getBalanceAmount(new BigNumber(bg._hex), token.decimals));
    }, {
        enabled,
        refetchOnMount: false,
        refetchInterval: 1000 * 10, // refetch every 10 seconds
    });
    if (!isNotSolOrIsNativeSol) {
        return solBalances ? (_a = solBalances[token === null || token === void 0 ? void 0 : token.address]) !== null && _a !== void 0 ? _a : BIG_ZERO : null;
    }
    return enabled ? data : null;
};
/**
 * This get the balance of all the tokens in the wallet
 */
export const useAllSolTokenBalance = () => {
    const { account, chainId, solanaWallet } = useActiveWeb3React();
    const { connection } = useConnection();
    const enabled = Boolean(isSolChain(chainId) && account);
    const { data } = useQuery(['solanaWalletBalances', account], () => {
        return connection
            .getParsedTokenAccountsByOwner(solanaWallet.publicKey, {
            programId: SOLANA_TOKEN_PROGRAM,
        })
            .then((response) => {
            var _a, _b, _c, _d, _e, _f, _g, _h;
            const balances = {};
            for (const value of response.value) {
                if (((_d = (_c = (_b = (_a = value === null || value === void 0 ? void 0 : value.account) === null || _a === void 0 ? void 0 : _a.data) === null || _b === void 0 ? void 0 : _b.parsed) === null || _c === void 0 ? void 0 : _c.info) === null || _d === void 0 ? void 0 : _d.mint) && ((_h = (_g = (_f = (_e = value === null || value === void 0 ? void 0 : value.account) === null || _e === void 0 ? void 0 : _e.data) === null || _f === void 0 ? void 0 : _f.parsed) === null || _g === void 0 ? void 0 : _g.info) === null || _h === void 0 ? void 0 : _h.mint)) {
                    balances[value.account.data.parsed.info.mint] = new BigNumber(value.account.data.parsed.info.tokenAmount.uiAmount);
                }
            }
            return balances;
        });
    }, {
        enabled,
        refetchOnMount: false,
        refetchInterval: 1000 * 10, // refetch every 10 seconds
    });
    return data;
};
/**
 * Get the wallet balance of a token
 * @param tokenAddress The token address to get the balance of
 * @returns The balance of the token in BigNumber (decimal parsed). Returns null if empty or loading
 */
export const useWalletBalanceByAddress = (tokenAddress) => {
    const token = useToken(tokenAddress);
    return useWalletBalance(token);
};
/**
 * Get the wallet balance of the ETH token
 * @returns The balance of ETH in BigNumber (decimal parsed). Returns null if empty or loading
 */
export const useETHBalance = () => {
    const { chainId } = useActiveWeb3React();
    const token = ETH[chainId];
    return useWalletBalance(token);
};
//# sourceMappingURL=useWalletBalance.js.map