import { getAddress } from '@ethersproject/address';
import { Contract } from '@ethersproject/contracts';
import { ChainId } from './constants';
import { provider } from '../wagmiUtil';
import { ETH, WETH } from './multichain';
import BigNumber from 'bignumber.js';
import { isSolChain } from './chainIdUtil';
/**
 * Truncate a transaction or address hash
 */
export const truncateHash = (address, startLength = 4, endLength = 4) => {
    return `${address.substring(0, startLength)}••••${address.substring(address.length - endLength)}`;
};
export const isValidSolanaTokenMint = (address) => {
    // Regular expression to match Solana token mint addresses
    const solanaTokenMintRegex = /^[1-9A-HJ-NP-Za-km-z]{32,44}$/;
    // Test the provided address against the regex
    try {
        return solanaTokenMintRegex.test(address) ? address : false;
    }
    catch (_a) {
        return false;
    }
};
export const isAddress = (value, chainId) => {
    if (isSolChain(chainId)) {
        return isValidSolanaTokenMint(value);
    }
    try {
        return getAddress(value);
    }
    catch (_a) {
        return false;
    }
};
// account is optional
export const getContract = (address, ABI, signer, chainId = ChainId.POLYGON) => {
    if (!isAddress(address, chainId)) {
        return null;
    }
    const signerOrProvider = signer !== null && signer !== void 0 ? signer : provider({ chainId });
    return new Contract(address, ABI, signerOrProvider);
};
/**
 * Get wrapped version of the token. If the token is not unwrapped, return the token itself
 * @param token The token to wrap
 * @returns
 */
export const wrappedToken = (token) => {
    if (token && token.address === ETH[token.chainId].address) {
        return WETH[token.chainId];
    }
    return token;
};
/**
 * Get unwrapped version of the token. If the token is not wrapped, return the token itself
 * @param token The token to unwrap
 * @returns
 */
export const unwrappedToken = (token) => {
    if (token.address === WETH[token.chainId].address) {
        return ETH[token.chainId];
    }
    return token;
};
/**
 * Get balance amount from a decimal amount
 * @example 1234500000000000000 => 1.2345
 * @param amount The amount to convert
 * @param decimals The decimals of the token
 * @returns
 */
export const getBalanceAmount = (amount, decimals) => {
    if (!amount || !decimals) {
        return null;
    }
    return amount.div(new BigNumber(10).pow(decimals));
};
/**
 * Get decimal amount from a balance amount
 * @example 1.2345 => 1234500000000000000
 * @param amount The amount to convert
 * @param decimals The decimals of the token
 * @returns
 */
export const getDecimalAmount = (amount, decimals) => {
    if (!amount || !decimals) {
        return null;
    }
    return amount.times(new BigNumber(10).pow(decimals));
};
/**
 * Get formatted amount
 * @example 11.2345 => 11.234; 0.00123 => 0.00123
 * @param amount The amount to convert
 * @returns
 */
export const getFormattedAmount = (amount, precision = 6, shouldCut = true) => {
    if (!amount) {
        return null;
    }
    if (amount.gt(100000)) {
        return amount.decimalPlaces(0, BigNumber.ROUND_UP).toString();
    }
    if (shouldCut && amount.gt(0) && amount.lt(0.0001)) {
        return toSubscriptZeros(amount);
    }
    return amount.precision(precision).toString();
};
/**
 * Get formatted amount with abbreviation
 * @returns
 * @example 1234567 => 1.23M
 */
export const formatNumberWithAbbrev = (num, decimals, prefix = '') => {
    if (num) {
        const suffixes = ['', 'K', 'M', 'B', 'T', 'Q'];
        const magnitude = Math.floor(Math.log10(Math.abs(num)) / 3);
        const rounded = (num / Math.pow(1000, magnitude)).toFixed(decimals);
        return `${prefix}${rounded}${suffixes[magnitude]}`;
    }
    else {
        return '';
    }
};
/**
 * Get formatted currency amount
 * @example 1111.2345 => $1,111.23; 0.00123 => $0.00123
 * @param amount The amount to convert
 * @returns
 */
export const getFormattedCurrencyAmount = (amount) => {
    if (!amount || amount.eq(0)) {
        return null;
    }
    if (amount.lt(1)) {
        if (amount.lt(0.0001)) {
            return `$${toSubscriptZeros(amount)}`;
        }
        return `$${amount.precision(4).toString()}`;
    }
    return amount
        .decimalPlaces(2, BigNumber.ROUND_UP)
        .toNumber()
        .toLocaleString('en-US', { style: 'currency', currency: 'USD' });
};
/**
 * Check if the token address is ETH
 * @param address address of the token
 */
export const isEth = (address, chainId) => address === ETH[chainId].address;
/**
 * Get pretty name
 * @param name
 * @example helloWorld -> Hello World
 */
export const getPrettyName = (name) => {
    if (!name)
        return '';
    const trimName = name.trim();
    return trimName
        ? (trimName[0].toUpperCase() + trimName.slice(1)).replace(/([A-Z][a-z]+|[A-Z]+(?=[A-Z]))/g, ' $1').trim()
        : '';
};
const subscriptMap = {
    '0': '₀',
    '1': '₁',
    '2': '₂',
    '3': '₃',
    '4': '₄',
    '5': '₅',
    '6': '₆',
    '7': '₇',
    '8': '₈',
    '9': '₉',
};
/**
 * Convert leading zeros in a number to subscript numbers
 * @example 0.00012345 => 0.0₄123
 */
export const toSubscriptZeros = (number) => {
    // Convert number to string
    const numberStr = number.toFixed(18);
    // Split the number into integer and fractional parts
    let [, fractionalPart] = numberStr.split('.');
    if (fractionalPart) {
        // Match leading zeros in the fractional part
        const leadingZeros = fractionalPart.match(/^0+/);
        if (leadingZeros) {
            // Count the number of leading zeros
            const zeroCount = leadingZeros[0].length;
            // Replace the leading zeros with the subscript number
            const subscriptZeros = zeroCount
                .toString()
                .split('')
                .map((digit) => subscriptMap[digit] || digit)
                .join('');
            // Remove the leading zeros from the fractional part
            fractionalPart = fractionalPart.replace(/^0+/, '');
            // Return the result in the format 0.0(subscript count)rest of the digits
            return `0.0${subscriptZeros}${fractionalPart.slice(0, 4).replace(/0+$/, '')}`;
        }
    }
};
//# sourceMappingURL=index.js.map